@@ -2,13 +2,16 @@
 from contextlib import suppress
 from enum import Enum
 from inspect import Signature, isawaitable, isclass
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, List, NoReturn, Optional, Type, Union, cast
 
 from pydantic import validate_arguments
-from starlette.responses import FileResponse, RedirectResponse
 from starlette.responses import Response as StarletteResponse
-from starlette.responses import StreamingResponse
-from starlette.status import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT
+from starlette.status import (
+    HTTP_200_OK,
+    HTTP_201_CREATED,
+    HTTP_204_NO_CONTENT,
+    HTTP_304_NOT_MODIFIED,
+)
 
 from starlite.constants import REDIRECT_STATUS_CODES
 from starlite.datastructures import (
@@ -17,9 +20,8 @@ from starlite.datastructures import (
     Cookie,
     File,
     Redirect,
+    ResponseContainer,
     ResponseHeader,
-    Stream,
-    Template,
 )
 from starlite.enums import HttpMethod, MediaType
 from starlite.exceptions import (
@@ -35,7 +37,7 @@ from starlite.lifecycle_hooks import (
 )
 from starlite.plugins import PluginProtocol, get_plugin_for_value
 from starlite.provide import Provide
-from starlite.response import Response, TemplateResponse
+from starlite.response import Response
 from starlite.types import (
     AfterRequestHandler,
     AfterResponseHandler,
@@ -261,6 +263,9 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         Returns all header parameters in the scope of the handler function
 
         This method is memoized so the computation occurs only once.
+
+        Returns:
+            A dictionary mapping keys to [ResponseHeader][starlite.datastructures.ResponseHeader] instances
         """
         if self._resolved_response_headers is Empty:
             self._resolved_response_headers = {}
@@ -273,6 +278,9 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         Returns a list of Cookie instances. Filters the list to ensure each cookie key is unique.
 
         This method is memoized so the computation occurs only once.
+
+        Returns:
+            A list of [Cookie][starlite.datastructures.Cookie] instances.
         """
         if self._resolved_response_cookies is Empty:
             self._resolved_response_cookies = []
@@ -289,7 +297,10 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         Resolves the before_handler handler by starting from the route handler and moving up.
 
         If a handler is found it is returned, otherwise None is set.
-        This method is memoized so the computation occurs only once
+        This method is memoized so the computation occurs only once.
+
+        Returns:
+            An optional [before request lifecycle hook handler][starlite.life_cycle.BeforeRequestHook]
         """
         if self._resolved_before_request is Empty:
             self._resolved_before_request = BeforeRequestHook.resolve_for_handler(self, "before_request")
@@ -300,7 +311,10 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         Resolves the after_request handler by starting from the route handler and moving up.
 
         If a handler is found it is returned, otherwise None is set.
-        This method is memoized so the computation occurs only once
+        This method is memoized so the computation occurs only once.
+
+        Returns:
+            An optional [after request lifecycle hook handler][starlite.life_cycle.AfterRequestHook]
         """
         if self._resolved_after_request is Empty:
             self._resolved_after_request = AfterRequestHook.resolve_for_handler(self, "after_request")
@@ -311,7 +325,10 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         Resolves the after_response handler by starting from the route handler and moving up.
 
         If a handler is found it is returned, otherwise None is set.
-        This method is memoized so the computation occurs only once
+        This method is memoized so the computation occurs only once.
+
+        Returns:
+            An optional [after response lifecycle hook handler][starlite.life_cycle.AfterResponseHook]
         """
         if self._resolved_after_response is Empty:
             self._resolved_after_response = AfterResponseHook.resolve_for_handler(self, "after_response")
@@ -320,53 +337,73 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
     @property
     def http_methods(self) -> List["Method"]:
         """
-        Returns a list of the RouteHandler's HttpMethod members
+        Returns:
+            A list of the RouteHandler's [HttpMethod][starlite.types.Method] strings
         """
         return cast("List[Method]", self.http_method if isinstance(self.http_method, list) else [self.http_method])
 
     async def to_response(self, data: Any, app: "Starlite", plugins: List[PluginProtocol]) -> StarletteResponse:
         """
-        Given a data kwarg, determine its type and return the appropriate response
+        Determines the correct response type and returns it
+
+        Args:
+            data: An arbitrary value that is either a [Response][starlite.response.Response] instance,
+                a Starlette response, a [ResponseContainer][starlite.datastructures.ResponseContainer]
+                or a value for a response body.
+            app: The [Starlite][starlite.app.Starlite] application instance
+            plugins: [plugins][starlite.plugins.base.PluginProtocol]
+
+        Returns:
+            A Starlette compatible Response object
         """
-        if isawaitable(data):
-            data = await data
+
+        data = await self._normalize_response_data(data=data, plugins=plugins)
+
         media_type = self.media_type.value if isinstance(self.media_type, Enum) else self.media_type
-        headers = {k: v.value for k, v in self.resolve_response_headers().items() if not v.documentation_only}
+        headers = self.resolve_response_headers()
         cookies = self.resolve_response_cookies()
-        response: StarletteResponse
-        if isinstance(data, (StarletteResponse, Redirect, File, Stream, Template)):
-            response = self._get_response_from_data(
-                app=app,
-                cookies=cookies,
-                data=data,
-                headers=headers,
-                media_type=media_type,
-            )
+
+        if isinstance(data, ResponseContainer):
+            headers = {**self._normalize_headers(headers), **data.headers}
+            response = data.to_response(app=app, headers=headers, status_code=self.status_code, media_type=media_type)
+        elif isinstance(data, StarletteResponse):
+            response = data
         else:
-            plugin = get_plugin_for_value(value=data, plugins=plugins)
-            if plugin:
-                if is_async_callable(plugin.to_dict):
-                    if isinstance(data, (list, tuple)):
-                        data = [await plugin.to_dict(datum) for datum in data]  # type: ignore
-                    else:
-                        data = await plugin.to_dict(data)  # type: ignore
-                else:
-                    if isinstance(data, (list, tuple)):
-                        data = [plugin.to_dict(datum) for datum in data]
-                    else:
-                        data = plugin.to_dict(data)
-            response_class = self.resolve_response_class()
-            response = response_class(
-                background=self.background,
-                content=data,
-                headers=headers,
-                media_type=media_type,
-                status_code=self.status_code,
-            )
-            for cookie in self._normalize_cookies(cookies, []):
+            response = self._create_response(data=data, headers=self._normalize_headers(headers), media_type=media_type)
+
+        if cookies:
+            if hasattr(data, "cookies"):
+                normalized_cookies = self._normalize_cookies(data.cookies, cookies)
+            else:
+                normalized_cookies = self._normalize_cookies(cookies, [])
+
+            for cookie in normalized_cookies:
                 response.set_cookie(**cookie)
+
         return await self._process_after_request_hook(response)
 
+    def _create_response(self, data: Any, headers: Dict[str, Any], media_type: Union[MediaType, str]) -> Response:
+        """
+        Creates a response based on the passed in data
+
+        Args:
+            data: Value for the response body
+            headers: A mapping of key value pairs to set as the response headers
+            media_type: A string or [MediaType][starlite.enums.MediaType] enum member
+
+        Returns: Response
+
+        """
+
+        response_class = self.resolve_response_class()
+        return response_class(
+            background=self.background,
+            content=data,
+            headers=headers,
+            media_type=media_type,
+            status_code=self.status_code,
+        )
+
     def _validate_handler_function(self) -> None:
         """
         Validates the route handler function once it is set by inspecting its return annotations
@@ -375,9 +412,16 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         signature = Signature.from_callable(cast("AnyCallable", self.fn))
         return_annotation = signature.return_annotation
         if return_annotation is Signature.empty:
-            raise ValidationException(
+            raise ImproperlyConfiguredException(
                 "A return value of a route handler function should be type annotated."
-                "If your function doesn't return a value or returns None, annotate it as returning None."
+                "If your function doesn't return a value or returns None, annotate it as returning 'NoReturn' or 'None' respectively."
+            )
+        if (
+            self.status_code < 200 or self.status_code in {HTTP_204_NO_CONTENT, HTTP_304_NOT_MODIFIED}
+        ) and return_annotation not in [NoReturn, None]:
+            raise ImproperlyConfiguredException(
+                "A status code 204, 304 or in the range below 200 does not support a response body."
+                "If the function should return a value, change the route handler status code to an appropriate value.",
             )
         if isclass(return_annotation):
             with suppress(TypeError):
@@ -393,6 +437,37 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         if "data" in signature.parameters and "GET" in self.http_methods:
             raise ImproperlyConfiguredException("'data' kwarg is unsupported for 'GET' request handlers")
 
+    @staticmethod
+    async def _normalize_response_data(data: Any, plugins: List[PluginProtocol]) -> Any:
+        """
+        Normalizes the response's data by awaiting any async values and resolving and plugins
+
+        Args:
+            data: An arbitrary value
+            plugins: A list of [plugins][starlite.plugins.base.PluginProtocol]
+
+        Returns:
+            Value for the response body
+        """
+        if isawaitable(data):
+            data = await data
+
+        if plugins:
+            plugin = get_plugin_for_value(value=data, plugins=plugins)
+            if plugin:
+                if is_async_callable(plugin.to_dict):
+                    if isinstance(data, (list, tuple)):
+                        data = [await plugin.to_dict(datum) for datum in data]  # type: ignore
+                    else:
+                        data = await plugin.to_dict(data)  # type: ignore
+                else:
+                    if isinstance(data, (list, tuple)):
+                        data = [plugin.to_dict(datum) for datum in data]
+                    else:
+                        data = plugin.to_dict(data)
+
+        return data
+
     @staticmethod
     def _normalize_cookies(local_cookies: List[Cookie], layered_cookies: List[Cookie]) -> List[Dict[str, Any]]:
         """
@@ -409,63 +484,19 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
                 normalized_cookies.append(cookie.dict(exclude_none=True, exclude={"documentation_only", "description"}))
         return normalized_cookies
 
-    def _get_response_from_data(
-        self,
-        app: "Starlite",
-        cookies: List[Cookie],
-        data: Union[StarletteResponse, Redirect, File, Stream, Template],
-        headers: dict,
-        media_type: Union[MediaType, str],
-    ) -> StarletteResponse:
+    @staticmethod
+    def _normalize_headers(headers: Dict[str, "ResponseHeader"]) -> Dict[str, Any]:
         """
-        Determines the correct response type to return given data
+        Given a dictionary of ResponseHeader, filters them and returns a dictionary of values
+
+        Args:
+            headers: A dictionary of [ResponseHeader][starlite.datastructures.ResponseHeader] values
+
+        Returns:
+            A string keyed dictionary of normalized values
+
         """
-        if isinstance(data, (Redirect, File, Stream, Template)):
-            headers.update(data.headers)
-            normalized_cookies = self._normalize_cookies(data.cookies, cookies)
-            if isinstance(data, Redirect):
-                response: "StarletteResponse" = RedirectResponse(
-                    headers=headers, status_code=self.status_code, url=data.path, background=data.background
-                )
-            elif isinstance(data, File):
-                response = FileResponse(
-                    background=data.background,
-                    filename=data.filename,
-                    headers=headers,
-                    media_type=media_type,
-                    path=data.path,
-                    stat_result=data.stat_result,
-                )
-            elif isinstance(data, Stream):
-                response = StreamingResponse(
-                    background=data.background,
-                    content=data.iterator,
-                    headers=headers,
-                    media_type=media_type,
-                    status_code=self.status_code,
-                )
-            else:
-                if not app.template_engine:
-                    raise ImproperlyConfiguredException("Template engine is not configured")
-                response = TemplateResponse(
-                    background=data.background,
-                    context=data.context,
-                    headers=headers,
-                    status_code=self.status_code,
-                    template_engine=app.template_engine,
-                    template_name=data.name,
-                )
-        elif isinstance(data, Response):
-            response = data
-            normalized_cookies = self._normalize_cookies(data.cookies, cookies)
-        else:
-            response = data
-            normalized_cookies = self._normalize_cookies(cookies, [])
-
-        for cookie in normalized_cookies:
-            response.set_cookie(**cookie)
-
-        return response
+        return {k: v.value for k, v in headers.items() if not v.documentation_only}
 
     async def _process_after_request_hook(
         self,
@@ -473,6 +504,12 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
     ) -> StarletteResponse:
         """
         Receives a response and handles after_request, if defined.
+
+        Args:
+            response: A Starlette compatible response object
+
+        Returns:
+            A Starlette compatible response object
         """
         after_request = self.resolve_after_request()
         if after_request:
