@@ -21,6 +21,7 @@ if TYPE_CHECKING:
         CacheConfig,
         CompressionConfig,
         CORSConfig,
+        CSRFConfig,
         OpenAPIConfig,
         StaticFilesConfig,
         TemplateConfig,
@@ -129,6 +130,7 @@ def create_test_client(
     cache_config: "CacheConfig" = DEFAULT_CACHE_CONFIG,
     compression_config: Optional["CompressionConfig"] = None,
     cors_config: Optional["CORSConfig"] = None,
+    csrf_config: Optional["CSRFConfig"] = None,
     dependencies: Optional[Dict[str, "Provide"]] = None,
     exception_handlers: Optional[Dict[Union[int, "Type[Exception]"], "ExceptionHandler"]] = None,
     guards: Optional[List["Guard"]] = None,
@@ -153,6 +155,7 @@ def create_test_client(
             cache_config=cache_config,
             compression_config=compression_config,
             cors_config=cors_config,
+            csrf_config=csrf_config,
             dependencies=dependencies,
             exception_handlers=exception_handlers,
             guards=guards,
